// **********************************************************************
//
// Copyright (c) 1998
// Object-Oriented Concepts, Inc.
// Billerica, MA, USA
//
// Copyright (c) 1998
// Object-Oriented Concepts GmbH
// Ettlingen, Germany
//
// All Rights Reserved
//
// **********************************************************************

#ifndef OB_CUSTOM_IMPL_H
#define OB_CUSTOM_IMPL_H

//
// Implementation of custom marshalling valuetypes
//

namespace CORBA
{

//
// IDL:omg.org/CORBA/DataOutputStream:1.0
//
class DataOutputStream_impl : virtual public DataOutputStream,
                              virtual public DefaultValueRefCountBase
{
    // NOT IMPLEMENTED
    DataOutputStream_impl(const DataOutputStream_impl&);
    void operator=(const DataOutputStream_impl&);

    OB::OutputStream& out_;

public:

    DataOutputStream_impl(OB::OutputStream& out);
    virtual ~DataOutputStream_impl();

    //
    // DataOutputStream cannot be copied or marshalled
    //
    virtual CORBA::ValueBase* _copy_value();
    virtual void _OB_marshalValue(OB::OutputStream&);
    virtual void _OB_marshalCountValue(OB::CountStream&);
    virtual void _OB_unmarshalValue(OB::InputStream&);

    //
    // IDL:omg.org/CORBA/DataOutputStream/write_any:1.0
    //
    virtual void write_any(const Any& value);

    //
    // IDL:omg.org/CORBA/DataOutputStream/write_boolean:1.0
    //
    virtual void write_boolean(Boolean value);

    //
    // IDL:omg.org/CORBA/DataOutputStream/write_char:1.0
    //
    virtual void write_char(Char value);

    //
    // IDL:omg.org/CORBA/DataOutputStream/write_wchar:1.0
    //
    virtual void write_wchar(WChar value);

    //
    // IDL:omg.org/CORBA/DataOutputStream/write_octet:1.0
    //
    virtual void write_octet(Octet value);

    //
    // IDL:omg.org/CORBA/DataOutputStream/write_short:1.0
    //
    virtual void write_short(Short value);

    //
    // IDL:omg.org/CORBA/DataOutputStream/write_ushort:1.0
    //
    virtual void write_ushort(UShort value);

    //
    // IDL:omg.org/CORBA/DataOutputStream/write_long:1.0
    //
    virtual void write_long(Long value);

    //
    // IDL:omg.org/CORBA/DataOutputStream/write_ulong:1.0
    //
    virtual void write_ulong(ULong value);

    //
    // IDL:omg.org/CORBA/DataOutputStream/write_longlong:1.0
    //
    virtual void write_longlong(LongLong value);

    //
    // IDL:omg.org/CORBA/DataOutputStream/write_ulonglong:1.0
    //
    virtual void write_ulonglong(ULongLong value);

    //
    // IDL:omg.org/CORBA/DataOutputStream/write_float:1.0
    //
    virtual void write_float(Float value);

    //
    // IDL:omg.org/CORBA/DataOutputStream/write_double:1.0
    //
    virtual void write_double(Double value);

    //
    // IDL:omg.org/CORBA/DataOutputStream/write_longdouble:1.0
    //
    virtual void write_longdouble(LongDouble value);

    //
    // IDL:omg.org/CORBA/DataOutputStream/write_string:1.0
    //
    virtual void write_string(const char* value);

    //
    // IDL:omg.org/CORBA/DataOutputStream/write_wstring:1.0
    //
    virtual void write_wstring(const wchar_t* value);

    //
    // IDL:omg.org/CORBA/DataOutputStream/write_Object:1.0
    //
    virtual void write_Object(Object_ptr value);

    //
    // IDL:omg.org/CORBA/DataOutputStream/write_Value:1.0
    //
    virtual void write_Value(ValueBase* value);

    //
    // IDL:omg.org/CORBA/DataOutputStream/write_TypeCode:1.0
    //
    virtual void write_TypeCode(TypeCode_ptr value);

    //
    // IDL:omg.org/CORBA/DataOutputStream/write_any_array:1.0
    //
    virtual void write_any_array(const AnySeq& seq,
                                 ULong offset,
                                 ULong length);

    //
    // IDL:omg.org/CORBA/DataOutputStream/write_boolean_array:1.0
    //
    virtual void write_boolean_array(const BooleanSeq& seq,
                                     ULong offset,
                                     ULong length);

    //
    // IDL:omg.org/CORBA/DataOutputStream/write_char_array:1.0
    //
    virtual void write_char_array(const CharSeq& seq,
                                  ULong offset,
                                  ULong length);

    //
    // IDL:omg.org/CORBA/DataOutputStream/write_wchar_array:1.0
    //
    virtual void write_wchar_array(const WCharSeq& seq,
                                   ULong offset,
                                   ULong length);

    //
    // IDL:omg.org/CORBA/DataOutputStream/write_octet_array:1.0
    //
    virtual void write_octet_array(const OctetSeq& seq,
                                   ULong offset,
                                   ULong length);

    //
    // IDL:omg.org/CORBA/DataOutputStream/write_short_array:1.0
    //
    virtual void write_short_array(const ShortSeq& seq,
                                   ULong offset,
                                   ULong length);

    //
    // IDL:omg.org/CORBA/DataOutputStream/write_ushort_array:1.0
    //
    virtual void write_ushort_array(const UShortSeq& seq,
                                    ULong offset,
                                    ULong length);

    //
    // IDL:omg.org/CORBA/DataOutputStream/write_long_array:1.0
    //
    virtual void write_long_array(const LongSeq& seq,
                                  ULong offset,
                                  ULong length);

    //
    // IDL:omg.org/CORBA/DataOutputStream/write_ulong_array:1.0
    //
    virtual void write_ulong_array(const ULongSeq& seq,
                                   ULong offset,
                                   ULong length);

    //
    // IDL:omg.org/CORBA/DataOutputStream/write_ulonglong_array:1.0
    //
    virtual void write_ulonglong_array(const ULongLongSeq& seq,
                                       ULong offset,
                                       ULong length);

    //
    // IDL:omg.org/CORBA/DataOutputStream/write_longlong_array:1.0
    //
    virtual void write_longlong_array(const LongLongSeq& seq,
                                      ULong offset,
                                      ULong length);

    //
    // IDL:omg.org/CORBA/DataOutputStream/write_float_array:1.0
    //
    virtual void write_float_array(const FloatSeq& seq,
                                   ULong offset,
                                   ULong length);

    //
    // IDL:omg.org/CORBA/DataOutputStream/write_double_array:1.0
    //
    virtual void write_double_array(const DoubleSeq& seq,
                                    ULong offset,
                                    ULong length);
};

//
// IDL:omg.org/CORBA/DataInputStream:1.0
//
class DataInputStream_impl : virtual public DataInputStream,
                             virtual public DefaultValueRefCountBase
{
    // NOT IMPLEMENTED
    DataInputStream_impl(const DataInputStream_impl&);
    void operator=(const DataInputStream_impl&);

    OB::InputStream& in_;

public:

    DataInputStream_impl(OB::InputStream& in);
    virtual ~DataInputStream_impl();

    //
    // DataInputStream cannot be copied or marshalled
    //
    virtual CORBA::ValueBase* _copy_value();
    virtual void _OB_marshalValue(OB::OutputStream&);
    virtual void _OB_marshalCountValue(OB::CountStream&);
    virtual void _OB_unmarshalValue(OB::InputStream&);

    //
    // IDL:omg.org/CORBA/DataInputStream/read_any:1.0
    //
    virtual Any* read_any();

    //
    // IDL:omg.org/CORBA/DataInputStream/read_boolean:1.0
    //
    virtual Boolean read_boolean();

    //
    // IDL:omg.org/CORBA/DataInputStream/read_char:1.0
    //
    virtual Char read_char();

    //
    // IDL:omg.org/CORBA/DataInputStream/read_wchar:1.0
    //
    virtual WChar read_wchar();

    //
    // IDL:omg.org/CORBA/DataInputStream/read_octet:1.0
    //
    virtual Octet read_octet();

    //
    // IDL:omg.org/CORBA/DataInputStream/read_short:1.0
    //
    virtual Short read_short();

    //
    // IDL:omg.org/CORBA/DataInputStream/read_ushort:1.0
    //
    virtual UShort read_ushort();

    //
    // IDL:omg.org/CORBA/DataInputStream/read_long:1.0
    //
    virtual Long read_long();

    //
    // IDL:omg.org/CORBA/DataInputStream/read_ulong:1.0
    //
    virtual ULong read_ulong();

    //
    // IDL:omg.org/CORBA/DataInputStream/read_longlong:1.0
    //
    virtual LongLong read_longlong();

    //
    // IDL:omg.org/CORBA/DataInputStream/read_ulonglong:1.0
    //
    virtual ULongLong read_ulonglong();

    //
    // IDL:omg.org/CORBA/DataInputStream/read_float:1.0
    //
    virtual Float read_float();

    //
    // IDL:omg.org/CORBA/DataInputStream/read_double:1.0
    //
    virtual Double read_double();

    //
    // IDL:omg.org/CORBA/DataInputStream/read_longdouble:1.0
    //
    virtual LongDouble read_longdouble();

    //
    // IDL:omg.org/CORBA/DataInputStream/read_string:1.0
    //
    virtual char* read_string();

    //
    // IDL:omg.org/CORBA/DataInputStream/read_wstring:1.0
    //
    virtual wchar_t* read_wstring();

    //
    // IDL:omg.org/CORBA/DataInputStream/read_Object:1.0
    //
    virtual Object_ptr read_Object();

    //
    // IDL:omg.org/CORBA/DataInputStream/read_Value:1.0
    //
    virtual ValueBase* read_Value();

    //
    // IDL:omg.org/CORBA/DataInputStream/read_TypeCode:1.0
    //
    virtual TypeCode_ptr read_TypeCode();

    //
    // IDL:omg.org/CORBA/DataInputStream/read_any_array:1.0
    //
    virtual void read_any_array(AnySeq& seq,
                                ULong offset,
                                ULong length);

    //
    // IDL:omg.org/CORBA/DataInputStream/read_boolean_array:1.0
    //
    virtual void read_boolean_array(BooleanSeq& seq,
                                    ULong offset,
                                    ULong length);

    //
    // IDL:omg.org/CORBA/DataInputStream/read_char_array:1.0
    //
    virtual void read_char_array(CharSeq& seq,
                                 ULong offset,
                                 ULong length);

    //
    // IDL:omg.org/CORBA/DataInputStream/read_wchar_array:1.0
    //
    virtual void read_wchar_array(WCharSeq& seq,
                                  ULong offset,
                                  ULong length);

    //
    // IDL:omg.org/CORBA/DataInputStream/read_octet_array:1.0
    //
    virtual void read_octet_array(OctetSeq& seq,
                                  ULong offset,
                                  ULong length);

    //
    // IDL:omg.org/CORBA/DataInputStream/read_short_array:1.0
    //
    virtual void read_short_array(ShortSeq& seq,
                                  ULong offset,
                                  ULong length);

    //
    // IDL:omg.org/CORBA/DataInputStream/read_ushort_array:1.0
    //
    virtual void read_ushort_array(UShortSeq& seq,
                                   ULong offset,
                                   ULong length);

    //
    // IDL:omg.org/CORBA/DataInputStream/read_long_array:1.0
    //
    virtual void read_long_array(LongSeq& seq,
                                 ULong offset,
                                 ULong length);

    //
    // IDL:omg.org/CORBA/DataInputStream/read_ulong_array:1.0
    //
    virtual void read_ulong_array(ULongSeq& seq,
                                  ULong offset,
                                  ULong length);

    //
    // IDL:omg.org/CORBA/DataInputStream/read_ulonglong_array:1.0
    //
    virtual void read_ulonglong_array(ULongLongSeq& seq,
                                      ULong offset,
                                      ULong length);

    //
    // IDL:omg.org/CORBA/DataInputStream/read_longlong_array:1.0
    //
    virtual void read_longlong_array(LongLongSeq& seq,
                                     ULong offset,
                                     ULong length);

    //
    // IDL:omg.org/CORBA/DataInputStream/read_float_array:1.0
    //
    virtual void read_float_array(FloatSeq& seq,
                                  ULong offset,
                                  ULong length);

    //
    // IDL:omg.org/CORBA/DataInputStream/read_double_array:1.0
    //
    virtual void read_double_array(DoubleSeq& seq,
                                   ULong offset,
                                   ULong length);
};

} // End of namespace CORBA

#endif
